@@ -42,213 +42,216 @@
 
 #define TRACKER_DEFAULT_CONTACT_ME "<urn:nco:default-contact-me>"
 
-#define CONTACTS_QUERY_ALL \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-		"?suffix ?email "					\
+#define CONTACTS_QUERY_ALL						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) "				\
+	"nco:nameAdditional(?c) nco:nameHonorificPrefix(?c) "		\
+	"nco:nameHonorificSuffix(?c) nco:emailAddress(?e) "		\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:fullname ?fullname } "			\
-	"OPTIONAL { ?contact nco:hasEmailAddress ?email } "		\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"OPTIONAL { "							\
+		"?c nco:hasAffiliation ?a . "				\
+		"?a nco:hasPhoneNumber ?w . "				\
+		"?c nco:hasEmailAddress ?e . "				\
+		"?c nco:hasPostalAddress ?p . "				\
+	"} "								\
 	"}"
 
-#define CONTACTS_QUERY_ALL_LIST \
-	"SELECT ?contact ?family ?given ?additional ?prefix "		\
-		"?suffix ?phone "					\
+#define CONTACTS_QUERY_ALL_LIST						\
+	"SELECT nco:contactUID(?c) nco:nameFamily(?c) "			\
+	"nco:nameGiven(?c) nco:nameAdditional(?c) "			\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c) "	\
+	"nco:phoneNumber(?h) "						\
 	"WHERE { "							\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
 	"}"
 
-#define MISSED_CALLS_QUERY \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-		"?suffix ?email "					\
+#define MISSED_CALLS_QUERY						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) "				\
+	"nco:nameAdditional(?c) nco:nameHonorificPrefix(?c) "		\
+	"nco:nameHonorificSuffix(?c) nco:emailAddress(?e) "		\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false ; "					\
 		"nmo:isAnswered false ."				\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:fullname ?fullname } "			\
-	"OPTIONAL { ?contact nco:hasEmailAddress ?email } "		\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"OPTIONAL { "							\
+		"?c nco:hasAffiliation ?a . "				\
+		"?a nco:hasPhoneNumber ?w . "				\
+		"?c nco:hasEmailAddress ?e . "				\
+		"?c nco:hasPostalAddress ?p . "				\
+	"} "								\
+	"} ORDER BY DESC(nmo:receivedDate(?call))"
 
-#define MISSED_CALLS_LIST \
-	"SELECT ?contact ?family ?given ?additional ?prefix "		\
-		"?suffix ?phone "					\
+#define MISSED_CALLS_LIST						\
+	"SELECT nco:contactUID(?c) nco:nameFamily(?c) "			\
+	"nco:nameGiven(?c) nco:nameAdditional(?c) "			\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c) "	\
+	"nco:phoneNumber(?h) "						\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false ; "					\
 		"nmo:isAnswered false ."				\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:hasEmailAddress ?email } "		\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"} ORDER BY DESC(nmo:receivedDate(?call))"
 
-#define INCOMING_CALLS_QUERY \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-		"?suffix ?email "					\
+#define INCOMING_CALLS_QUERY						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) "				\
+	"nco:nameAdditional(?c) nco:nameHonorificPrefix(?c) "		\
+	"nco:nameHonorificSuffix(?c) nco:emailAddress(?e) "		\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:fullname ?fullname } "			\
-	"OPTIONAL { ?contact nco:hasEmailAddress ?email } "		\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"OPTIONAL { "							\
+		"?c nco:hasAffiliation ?a . "				\
+		"?a nco:hasPhoneNumber ?w . "				\
+		"?c nco:hasEmailAddress ?e . "				\
+		"?c nco:hasPostalAddress ?p . "				\
+	"} "								\
+	"} ORDER BY DESC(nmo:receivedDate(?call))"
 
 #define INCOMING_CALLS_LIST						\
-	"SELECT ?contact ?family ?given ?additional ?prefix "		\
-		"?suffix ?phone "					\
+	"SELECT nco:contactUID(?c) nco:nameFamily(?c) "			\
+	"nco:nameGiven(?c) nco:nameAdditional(?c) "			\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c) "	\
+	"nco:phoneNumber(?h) "						\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"} ORDER BY DESC(nmo:receivedDate(?call))"
 
-#define OUTGOING_CALLS_QUERY \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-		"?suffix ?email "					\
+#define OUTGOING_CALLS_QUERY						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) "				\
+	"nco:nameAdditional(?c) nco:nameHonorificPrefix(?c) "		\
+	"nco:nameHonorificSuffix(?c) nco:emailAddress(?e) "		\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:to ?contact ; "					\
+		"nmo:to ?c ; "						\
 		"nmo:isSent true . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:fullname ?fullname } "			\
-	"OPTIONAL { ?contact nco:hasEmailAddress ?email } "		\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"OPTIONAL { "							\
+		"?c nco:hasAffiliation ?a . "				\
+		"?a nco:hasPhoneNumber ?w . "				\
+		"?c nco:hasEmailAddress ?e . "				\
+		"?c nco:hasPostalAddress ?p . "				\
+	"} "								\
+	"} ORDER BY DESC(nmo:sentDate(?call))"
 
-#define OUTGOING_CALLS_LIST \
-	"SELECT ?contact ?family ?given ?additional ?prefix "		\
-		"?suffix ?phone "					\
+#define OUTGOING_CALLS_LIST						\
+	"SELECT nco:contactUID(?c) nco:nameFamily(?c) "			\
+	"nco:nameGiven(?c) nco:nameAdditional(?c) "			\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c) "	\
+	"nco:phoneNumber(?h) "						\
 	"WHERE { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:to ?contact ; "					\
+		"nmo:to ?c ; "						\
 		"nmo:isSent true . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { ?contact nco:nameFamily ?family } "			\
-	"OPTIONAL { ?contact nco:nameGiven ?given } "			\
-	"OPTIONAL { ?contact nco:nameAdditional ?additional } "		\
-	"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "	\
-	"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "	\
-	"}"
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"} ORDER BY DESC(nmo:sentDate(?call))"
 
-#define COMBINED_CALLS_QUERY \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-		"?suffix ?email "					\
+#define COMBINED_CALLS_QUERY						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) "				\
+	"nco:nameAdditional(?c) nco:nameHonorificPrefix(?c) "		\
+	"nco:nameHonorificSuffix(?c) nco:emailAddress(?e) "		\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
 	"{ "								\
 		"?call a nmo:Call ; "					\
-		"nmo:to ?contact ; "					\
+		"nmo:to ?c ; "						\
 		"nmo:isSent true . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-		"OPTIONAL { ?contact nco:nameFamily ?family } "		\
-		"OPTIONAL { ?contact nco:nameGiven ?given } "		\
-		"OPTIONAL { ?contact nco:fullname ?fullname } "		\
-		"OPTIONAL { ?contact nco:hasEmailAddress ?email } "	\
-		"OPTIONAL { ?contact nco:nameAdditional ?additional } "	\
-		"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "\
-		"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+		"OPTIONAL { "						\
+			"?c nco:hasAffiliation ?a . "			\
+			"?a nco:hasPhoneNumber ?w . "			\
+			"?c nco:hasEmailAddress ?e . "			\
+			"?c nco:hasPostalAddress ?p . "			\
+		"} "							\
 	"} UNION { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-		"OPTIONAL { ?contact nco:nameFamily ?family } "		\
-		"OPTIONAL { ?contact nco:nameGiven ?given } "		\
-		"OPTIONAL { ?contact nco:fullname ?fullname } "		\
-		"OPTIONAL { ?contact nco:hasEmailAddress ?email } "	\
-		"OPTIONAL { ?contact nco:nameAdditional ?additional } "	\
-		"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "\
-		"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+		"OPTIONAL { "						\
+			"?c nco:hasAffiliation ?a . "			\
+			"?a nco:hasPhoneNumber ?w . "			\
+			"?c nco:hasEmailAddress ?e . "			\
+			"?c nco:hasPostalAddress ?p . "			\
+		"} "							\
 	"} } "
 
-#define COMBINED_CALLS_LIST \
-	"SELECT ?contact ?family ?given ?additional ?prefix "		\
-		"?suffix ?phone "					\
+#define COMBINED_CALLS_LIST						\
+	"SELECT nco:contactUID(?c) nco:nameFamily(?c) "			\
+	"nco:nameGiven(?c) nco:nameAdditional(?c) "			\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c) "	\
+	"nco:phoneNumber(?h) "						\
 	"WHERE { "							\
 	"{ "								\
 		"?call a nmo:Call ; "					\
-		"nmo:to ?contact ; "					\
+		"nmo:to ?c ; "						\
 		"nmo:isSent true . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-		"OPTIONAL { ?contact nco:nameFamily ?family } "		\
-		"OPTIONAL { ?contact nco:nameGiven ?given } "		\
-		"OPTIONAL { ?contact nco:nameAdditional ?additional } "	\
-		"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "\
-		"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
 	"} UNION { "							\
 		"?call a nmo:Call ; "					\
-		"nmo:from ?contact ; "					\
+		"nmo:from ?c ; "					\
 		"nmo:isSent false . "					\
-		"?contact a nco:PersonContact ; "			\
-		"nco:hasPhoneNumber ?phone . "				\
-		"OPTIONAL { ?contact nco:nameFamily ?family } "		\
-		"OPTIONAL { ?contact nco:nameGiven ?given } "		\
-		"OPTIONAL { ?contact nco:nameAdditional ?additional } "	\
-		"OPTIONAL { ?contact nco:nameHonorificPrefix ?prefix } "\
-		"OPTIONAL { ?contact nco:nameHonorificSuffix ?suffix } "\
+		"?c a nco:PersonContact ; "				\
+		"nco:hasPhoneNumber ?h . "				\
 	"} } "
 
 
-#define CONTACTS_QUERY_FROM_URI \
-	"SELECT ?phone ?fullname ?family ?given ?additional ?prefix "	\
-	"	?suffix ?email "					\
+#define CONTACTS_QUERY_FROM_URI						\
+	"SELECT nco:phoneNumber(?h) nco:fullname(?c) "			\
+	"nco:nameFamily(?c) nco:nameGiven(?c) nco:nameAdditional(?c) "	\
+	"nco:nameHonorificPrefix(?c) nco:nameHonorificSuffix(?c)  "	\
+	"nco:emailAddress(?e) "						\
+	"nco:phoneNumber(?w) nco:pobox(?p) nco:extendedAddress(?p) "	\
+	"nco:streetAddress(?p) nco:locality(?p) nco:region(?p) "	\
+	"nco:postalcode(?p) nco:country(?p) "				\
 	"WHERE { "							\
-		"<%s> a nco:PersonContact ; "				\
-		"nco:hasPhoneNumber ?phone . "				\
-	"OPTIONAL { <%s> nco:nameFamily ?family } "			\
-	"OPTIONAL { <%s> nco:nameGiven ?given } "			\
-	"OPTIONAL { <%s> nco:fullname ?fullname } "			\
-	"OPTIONAL { <%s> nco:hasEmailAddress ?email } "			\
-	"OPTIONAL { <%s> nco:nameAdditional ?additional } "		\
-	"OPTIONAL { <%s> nco:nameHonorificPrefix ?prefix } "		\
-	"OPTIONAL { <%s> nco:nameHonorificSuffix ?suffix } "		\
+		"?c a nco:PersonContact ; "				\
+		"nco:contactUID <%s> ; "				\
+		"nco:hasPhoneNumber ?h . "				\
+	"OPTIONAL { "							\
+		"?c nco:hasAffiliation ?a . "				\
+		"?a nco:hasPhoneNumber ?w . "				\
+		"?c nco:hasEmailAddress ?e . "				\
+		"?c nco:hasPostalAddress ?p . "				\
+	"} "								\
 	"}"
 
 typedef void (*reply_list_foreach_t) (char **reply, int num_fields,
@@ -265,6 +268,7 @@ struct phonebook_data {
 	phonebook_cb cb;
 	void *user_data;
 	int index;
+	gboolean vcardentry;
 	const struct apparam_field *params;
 };
 
@@ -427,7 +431,7 @@ done:
 	g_free(pending);
 }
 
-static int query_tracker(const char* query, int num_fields,
+static int query_tracker(const char *query, int num_fields,
 				reply_list_foreach_t callback, void *user_data)
 {
 	struct pending_reply *pending;
@@ -468,16 +472,23 @@ static void pull_contacts(char **reply, int num_fields, void *user_data)
 	struct phonebook_data *data = user_data;
 	const struct apparam_field *params = data->params;
 	struct phonebook_contact *contact;
+	struct phonebook_number *number;
 	GString *vcards = data->vcards;
 	int last_index;
 
+	DBG("reply %p", reply);
+
 	if (reply == NULL)
 		goto done;
 
+	/* We are doing a PullvCardEntry, no need for those checks */
+	if (data->vcardentry)
+		goto add_entry;
+
 	data->index++;
 
 	/* Just interested in knowing the phonebook size */
-	if (params->maxlistcount == 0)
+	if (!data->vcardentry && params->maxlistcount == 0)
 		return;
 
 	last_index = params->liststartoffset + params->maxlistcount;
@@ -485,9 +496,8 @@ static void pull_contacts(char **reply, int num_fields, void *user_data)
 	if (data->index < params->liststartoffset || data->index > last_index)
 		return;
 
+add_entry:
 	contact = g_new0(struct phonebook_contact, 1);
-	contact->tel = g_strdup(reply[0]);
-	contact->tel_type = 1; /* HOME */
 	contact->fullname = g_strdup(reply[1]);
 	contact->family = g_strdup(reply[2]);
 	contact->given = g_strdup(reply[3]);
@@ -495,6 +505,30 @@ static void pull_contacts(char **reply, int num_fields, void *user_data)
 	contact->prefix = g_strdup(reply[5]);
 	contact->suffix = g_strdup(reply[6]);
 	contact->email = g_strdup(reply[7]);
+	contact->pobox = g_strdup(reply[9]);
+	contact->extended = g_strdup(reply[10]);
+	contact->street = g_strdup(reply[11]);
+	contact->locality = g_strdup(reply[12]);
+	contact->region = g_strdup(reply[13]);
+	contact->postal = g_strdup(reply[14]);
+	contact->country = g_strdup(reply[15]);
+
+	number = g_new0(struct phonebook_number, 1);
+	number->tel = g_strdup(reply[0]);
+	number->type = 0; /* HOME */
+
+	contact->numbers = g_slist_append(contact->numbers, number);
+
+	/* Has WORK Phonenumber */
+	if (strlen(reply[8])) {
+		number = g_new0(struct phonebook_number, 1);
+		number->tel = g_strdup(reply[8]);
+		number->type = 3; /* WORK */
+
+		contact->numbers = g_slist_append(contact->numbers, number);
+	}
+
+	DBG("contact %p", contact);
 
 	phonebook_add_contact(vcards, contact, params->filter, params->format);
 	phonebook_contact_free(contact);
@@ -507,6 +541,7 @@ done:
 							data->user_data);
 
 	g_string_free(vcards, TRUE);
+	g_free(data);
 }
 
 static void add_to_cache(char **reply, int num_fields, void *user_data)
@@ -630,9 +665,11 @@ int phonebook_pull(const char *name, const struct apparam_field *params,
 	struct phonebook_data *data;
 	const char *query;
 
+	DBG("name %s", name);
+
 	query = name2query(name);
 	if (query == NULL)
-		return -1;
+		return -ENOENT;
 
 	data = g_new0(struct phonebook_data, 1);
 	data->vcards = g_string_new(NULL);
@@ -640,7 +677,7 @@ int phonebook_pull(const char *name, const struct apparam_field *params,
 	data->user_data = user_data;
 	data->cb = cb;
 
-	return query_tracker(query, 8, pull_contacts, data);
+	return query_tracker(query, 16, pull_contacts, data);
 }
 
 int phonebook_get_entry(const char *folder, const char *id,
@@ -651,16 +688,18 @@ int phonebook_get_entry(const char *folder, const char *id,
 	char *query;
 	int ret;
 
+	DBG("folder %s id %s", folder, id);
+
 	data = g_new0(struct phonebook_data, 1);
 	data->vcards = g_string_new(NULL);
 	data->user_data = user_data;
 	data->params = params;
 	data->cb = cb;
+	data->vcardentry = TRUE;
 
-	query = g_strdup_printf(CONTACTS_QUERY_FROM_URI,
-					id, id, id, id, id, id, id, id);
+	query = g_strdup_printf(CONTACTS_QUERY_FROM_URI, id);
 
-	ret = query_tracker(query, 8, pull_contacts, data);;
+	ret = query_tracker(query, 16, pull_contacts, data);
 
 	g_free(query);
 
@@ -673,9 +712,11 @@ int phonebook_create_cache(const char *name, phonebook_entry_cb entry_cb,
 	struct cache_data *cache;
 	const char *query;
 
+	DBG("name %s", name);
+
 	query = folder2query(name);
 	if (query == NULL)
-		return -1;
+		return -ENOENT;
 
 	cache = g_new0(struct cache_data, 1);
 	cache->entry_cb = entry_cb;
